///////////////////////////////////////////////////////////////////////////////
// Filename: networkclass.cpp
///////////////////////////////////////////////////////////////////////////////
#include "networkclass.h"


NetworkClass::NetworkClass()
{
	m_online = false;
}


NetworkClass::NetworkClass(const NetworkClass& other)
{
}


NetworkClass::~NetworkClass()
{
}


bool NetworkClass::Initialize()
{
	bool result;


	// Initialize winsock for using window's sockets.
	result = InitializeWinSock();
	if(!result)
	{
		return false;
	}

	return true;
}


void NetworkClass::Shutdown()
{
	// If it never went online then no need to shut it down.
	if(!m_online)
	{
		return;
	}

	// Send a message to the server letting it know this client is disconnecting.
	SendDisconnectMessage();

	// Set the client to be offline.
	m_online = false;

	// Wait for the network I/O thread to complete.
	while(m_threadActive)
	{
	}

	// Close the socket.
	closesocket(m_clientSocket);

	// Shutdown winsock.
	ShutdownWinsock();

	return;
}


void NetworkClass::Frame()
{
	ProcessLatency();

	return;
}


bool NetworkClass::InitializeWinSock()
{
	unsigned short version;      
	WSADATA wsaData;
	int error;
	unsigned long bufferSize;
	WSAPROTOCOL_INFOW* protocolBuffer;  
	int protocols[2];


	// Create a 2.0 macro to check versions.
	version = MAKEWORD(2, 0);
	
	// Get the data to see if it handles the version we want.
	error = WSAStartup(version, &wsaData);
    if(error != 0)
	{
		return false;
	}
	
	// Check to see if the winsock dll is version 2.0.
	if((LOBYTE(wsaData.wVersion) != 2) || (HIBYTE(wsaData.wVersion) != 0))
	{
		return false;
	}

	// Request the buffer size needed for holding the protocols available.
	WSAEnumProtocols(NULL, NULL, &bufferSize);

	// Create a buffer for the protocol information structs.
	protocolBuffer = new WSAPROTOCOL_INFOW[bufferSize];
	if(!protocolBuffer)
	{
		return false;
	}

	// Create the list of protocols we are looking for which are TCP and UDP.
	protocols[0] = IPPROTO_TCP;
	protocols[1] = IPPROTO_UDP;

	// Retrieve information about available transport protocols, if no socket error then the protocols from the list will work.
	error = WSAEnumProtocols(protocols, protocolBuffer, &bufferSize);
	if(error == SOCKET_ERROR)
	{
		return false;
	}

	// Release the protocol buffer.
	delete [] protocolBuffer;
	protocolBuffer = 0;

	return true;
}


void NetworkClass::ShutdownWinsock()
{
	WSACleanup();
	return;
}


bool NetworkClass::ConnectToServer(char* ipAddress, unsigned short portNumber)
{
	unsigned long setting, inetAddress, startTime, threadId;
	MSG_GENERIC_DATA connectMessage, *message;
	int error, bytesSent, bytesRead;
	bool done, gotId;
	char recvBuffer[4096];
	HANDLE threadHandle;


    // Create a UDP socket.
	m_clientSocket = socket(AF_INET, SOCK_DGRAM, 0);
	if(m_clientSocket == INVALID_SOCKET)
	{
		return false;
	}

	// Set the client socket to non-blocking I/O.
	setting = 1;
	error = ioctlsocket(m_clientSocket, FIONBIO, &setting);
	if(error == SOCKET_ERROR)
	{
		return false;
	}

	// Save the size of the server address structure.
	m_addressLength = sizeof(m_serverAddress);

	// Setup the address of the server we are sending data to.
	inetAddress = inet_addr(ipAddress);
	memset((char*)&m_serverAddress, 0, m_addressLength);
	m_serverAddress.sin_family = AF_INET;
	m_serverAddress.sin_port = htons(portNumber);
	m_serverAddress.sin_addr.s_addr = inetAddress;

	// Setup a connect message to send to the server.
	connectMessage.type = MSG_CONNECT;
	connectMessage.toId = ID_SERVER;
	connectMessage.fromId = ID_UNKNOWN;

	// Send the connect message to the server.
	bytesSent = sendto(m_clientSocket, (char*)&connectMessage, sizeof(MSG_GENERIC_DATA), 0, (struct sockaddr*)&m_serverAddress, m_addressLength);
	if(bytesSent < 0)
	{
		return false;
	}

	// Record the time when the connect packet was sent.
	startTime = timeGetTime();

	// Set the boolean loop values.
	done = false;
	gotId = false;

	while(!done)
	{
		// Check for a reply message from the server.
		bytesRead = recvfrom(m_clientSocket, recvBuffer, 4096, 0, (struct sockaddr*)&m_serverAddress, &m_addressLength);
		if(bytesRead > 0)
		{
			done = true;
			gotId = true;
		}

		// Check to see if this loop has been running for longer than 2 seconds.
		if(timeGetTime() > (startTime + 2000))
		{
			done = true;
			gotId = false;
		}
	}

	// If it didn't get an ID in 2 seconds then the server was not up.
	if(!gotId)
	{
		return false;
	}

	// Coerce the message into a generic message type.
	message = (MSG_GENERIC_DATA*)recvBuffer;

	// Ensure it was an ID message.
	if(message->type != MSG_ID)
	{
		return false;
	}

	// Ensure the ID number was not a server full message.
	if(message->toId == -1)
	{
		return false;
	}

	// Store the ID number for this client for all future communication with the server.
	m_idNumber = message->toId;

	// Set the client to be online now.
	m_online = true;
	
	// Initialize the thread activity variable.
	m_threadActive = false;

	// Create a thread to listen for network I/O from the server.
	threadHandle = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)NetworkReadFunction, (void*)this, 0, &threadId);
	if(threadHandle == NULL)
	{
		return false;
	}

	// Initialize the network latency variables.
	m_latency = 0;
	m_pingTime = timeGetTime();

	return true;
}


int NetworkClass::GetLantency()
{
	return m_latency;
}


void NetworkClass::SetThreadActive()
{
	m_threadActive = true;
	return;
}


void NetworkClass::SetThreadInactive()
{
	m_threadActive = false;
	return;
}


bool NetworkClass::Online()
{
	return m_online;
}


SOCKET NetworkClass::GetClientSocket()
{
	return m_clientSocket;
}


void NetworkReadFunction(void* ptr)
{
	NetworkClass* networkClassPtr;
	struct sockaddr_in serverAddress;
	int addressLength;
	int bytesRead;
	char recvBuffer[4096];


	// Get a pointer to the calling object.
	networkClassPtr = (NetworkClass*)ptr;

	// Notify parent object that this thread is now active.
	networkClassPtr->SetThreadActive();

	// Set the size of the address.
	addressLength = sizeof(serverAddress);

	// Loop and read network messages while the client is online.
	while(networkClassPtr->Online())
	{
	    // Check if there is a message from the server.
		bytesRead = recvfrom(networkClassPtr->GetClientSocket(), recvBuffer, 4096, 0, (struct sockaddr*)&serverAddress, &addressLength);
		if(bytesRead > 0)
		{
			networkClassPtr->ReadNetworkMessage(recvBuffer);
		}
	}

	// Notify parent object that this thread is now inactive.
	networkClassPtr->SetThreadInactive();

	return;
}


void NetworkClass::ReadNetworkMessage(char* recvBuffer)
{
	MSG_GENERIC_DATA* message;

	
	// Coerce the message into a generic format to read the type of message.
	message = (MSG_GENERIC_DATA*)recvBuffer;

	switch(message->type)
	{
		case MSG_PING:
		{
			HandlePingMessage();
			break;
		}
		default:
		{
			break;
		}
	}

	return;
}


void NetworkClass::ProcessLatency()
{
	// If 5 seconds is up then send a ping message to the server.
	if(timeGetTime() >= (m_pingTime + 5000))
	{
		m_pingTime = timeGetTime();
		SendPing();
	}

	return;
}


void NetworkClass::SendPing()
{
	MSG_GENERIC_DATA message;
	int bytesSent;

	
	// Create the ping message.
	message.type = MSG_PING;
	message.toId = ID_SERVER;
	message.fromId = m_idNumber;

	bytesSent = sendto(m_clientSocket, (char*)&message, sizeof(MSG_GENERIC_DATA), 0, (struct sockaddr*)&m_serverAddress, m_addressLength);
	if(bytesSent != sizeof(MSG_GENERIC_DATA))
	{

	}

	return;
}


void NetworkClass::HandlePingMessage()
{
	m_latency = timeGetTime() - m_pingTime;
	return;
}


void NetworkClass::SendDisconnectMessage()
{
	MSG_GENERIC_DATA message;
	int bytesSent;

	
	// Create the ping message.
	message.type = MSG_DISCONNECT;
	message.toId = ID_SERVER;
	message.fromId = m_idNumber;

	bytesSent = sendto(m_clientSocket, (char*)&message, sizeof(MSG_GENERIC_DATA), 0, (struct sockaddr*)&m_serverAddress, m_addressLength);
	if(bytesSent != sizeof(MSG_GENERIC_DATA))
	{

	}

	return;
}